<?php

declare(strict_types=1);

namespace Drupal\travolute\Model;

use Carbon\Carbon;

class SearchRequest
{
    private $departureDate;
    private $departureOffset;
    private $query;
    private $minimumNumberOfStars;
    private $durations;
    private $boardTypes;
    private $occupancy;
    private $offset;
    private $limit;
    private $airport;

    /**
     * SearchRequest constructor.
     *
     * @param Carbon      $departureDate
     * @param array|null  $airport
     * @param int|null    $departureOffset
     * @param string|null $query
     * @param int|null    $minimumNumberOfStars
     * @param array|null  $durations
     * @param array|null  $boardTypes
     * @param int         $occupancy
     * @param int         $offset
     * @param int         $limit
     */
    public function __construct(
        Carbon $departureDate,
        array $airport = null,
        int $departureOffset = 0,
        string $query = null,
        int $minimumNumberOfStars = null,
        array $durations = null,
        array $boardTypes = null,
        int $occupancy = 2,
        int $offset = 0,
        int $limit = 20
    ) {
        $this->departureDate = $departureDate;
        $this->departureOffset = $departureOffset;
        $this->query = $query;
        $this->minimumNumberOfStars = $minimumNumberOfStars;
        $this->durations = $durations;
        $this->boardTypes = $boardTypes;
        $this->occupancy = $occupancy;
        $this->offset = $offset;
        $this->limit = $limit;
        $this->airport = $airport;
    }

    /**
     * @return Carbon
     */
    public function getDepartureDate(): Carbon
    {
        return $this->departureDate;
    }

    /**
     * @return int
     */
    public function getDepartureOffset(): int
    {
        return $this->departureOffset;
    }

    /**
     * @return mixed
     */
    public function getQuery()
    {
        return $this->query;
    }

    /**
     * @return mixed
     */
    public function getMinimumNumberOfStars()
    {
        return $this->minimumNumberOfStars;
    }

    /**
     * @return mixed
     */
    public function getDurations()
    {
        return $this->durations;
    }

    /**
     * @return array
     */
    public function getBoardTypes(): array
    {
        return $this->boardTypes;
    }

    /**
     * @return int
     */
    public function getOccupancy(): int
    {
        return $this->occupancy;
    }

    /**
     * @return int
     */
    public function getOffset(): int
    {
        return $this->offset;
    }

    /**
     * @return int
     */
    public function getLimit(): int
    {
        return $this->limit;
    }

    /**
     * @return array|null
     */
    public function getAirport(): ?array
    {
        return $this->airport;
    }

    /**
     * @param array|null $airport
     */
    public function setAirport(?array $airport): void
    {
        $this->airport = $airport;
    }


    /**
     * @param int|null $departureOffset
     */
    public function setDepartureOffset(int $departureOffset)
    {
        $this->departureOffset = $departureOffset;
    }

    /**
     * @param null|string $query
     */
    public function setQuery(string $query)
    {
        $this->query = $query;
    }

    /**
     * @param int|null $minimumNumberOfStars
     */
    public function setMinimumNumberOfStars(int $minimumNumberOfStars)
    {
        $this->minimumNumberOfStars = $minimumNumberOfStars;
    }

    /**
     * @param array|null $durations
     */
    public function setDurations(array $durations)
    {
        $this->durations = $durations;
    }

    /**
     * @param array|null $boardTypes
     */
    public function setBoardTypes(array $boardTypes)
    {
        $this->boardTypes = $boardTypes;
    }

    /**
     * @param int $occupancy
     */
    public function setOccupancy(int $occupancy)
    {
        $this->occupancy = $occupancy;
    }

    /**
     * @param int $offset
     */
    public function setOffset(int $offset)
    {
        $this->offset = $offset;
    }

    /**
     * @param int $limit
     */
    public function setLimit(int $limit)
    {
        $this->limit = $limit;
    }

    /**
     * Converts the object to be used to an array for Travolute Service
     *
     * @param void
     * @return array
     */
    public function __toArray(): array
    {
        $output = [
            'departure_date' => $this->getDepartureDate()->format('Y-m-d'),
            'departure_offset' => $this->getDepartureOffset(),
            'occupancy' => $this->getOccupancy(),
            'offset' => $this->getOffset(),
            'limit' => $this->getLimit(),
        ];

        if (\is_string($this->query)) {
            $output['query'] = $this->getQuery();
        }

        if (\is_int($this->minimumNumberOfStars)) {
            $output['stars'] = $this->getMinimumNumberOfStars();
        }

        if (\is_array($this->durations)) {
            $output['duration'] = $this->getDurations();
        }

        if (\is_array($this->airport)) {
            $output['airport'] = $this->getAirport();
        }

        if (\is_array($this->boardTypes)) {
            $output['boardtype'] = $this->getBoardTypes();
        }

        return $output;
    }
}
